""" corkscrew.comet
"""

import time
import demjson
import threading

import flask_sijax

from goulash.stdout import ThreadedStdout
from goulash.ansi import Ansi2HTMLConverter

from corkscrew.views import View

class SijaxView(View):
    """ Very basic corkscrew integration with sijax.
        This doesn't do much, you probably want to use
        the CometWorker class.
    """

    @property
    def sijax(self):
        from flask import g
        return g.sijax

    @property
    def is_sijax(self):
        return self.sijax.is_sijax_request

    def install_into_app(self, app):
        flask_sijax.route(self.app, self.url)(self.main)
        return []

class CometBase(SijaxView):
    pause_for = .25
    extra_scripts = [ '/static/js/sijax/sijax.js',
                      '/static/js/sijax/sijax_comet.js',
                      ]

    def get_busy(self, obj_response):
        return [
            lambda: obj_response.css('#loading_icon', 'display', 'none'),
            lambda: obj_response.css('#loading_icon', 'display', 'block')]

    def main(self):
        """
        {%extends "layout.html" %}
        {%block body%}
        {{extra_html|safe}}
        <script type="text/javascript">
        {{ g.sijax.get_js()|safe }}
        </script>

        <table><tr>
        <td>{%if not autostart%}<button id="btnStart">Start</button>{%endif%}</td>
        <td><img id="loading_icon" src="{{loading_icon}}"></td>
        <td>
        <div id="comet_data" style=""></div></td></tr></table>
        <script type="text/javascript">
        var data='{{data}}';
        {%if not autostart%}
        $('#btnStart').bind('click', function(){sjxComet.request('do_work', [data]);});
        {%else%}
        $(document).ready(function(){sjxComet.request('do_work', [data]);});
        {%endif%}
        $('#loading_icon').hide();
        </script>
        {%endblock%}
        """
        self.request_data = dict(self.request.args.items())
        if self.is_sijax:
            self.sijax.register_comet_callback('do_work', self.comet_handler)
            out = self.sijax.process_request()
            return out
        else:
            extra_html = getattr(self, 'extra_html', '')
            if callable(extra_html): extra_html = extra_html()
            return self.render(
                CometWorker.main.__doc__,
                extra_html=extra_html,
                javascript=self.sijax.get_js(),
                autostart=self['start'],
                data = demjson.encode(self.request_data),
                loading_icon='/static/img/loading.gif',)

    def comet_handler(self, obj_response, data):
        """ """
        data = demjson.decode(data)
        obj_response.html_append('#comet_data', self.style)
        hidebusy, showbusy = self.get_busy(obj_response)
        self.before_streaming(obj_response, data)
        while self.streaming:
            zult = self._read()
            if zult:
                zult = self.conv.convert(zult) # unansi the text
                obj_response.html_append('#comet_data', zult)
                hidebusy()
            else:
                showbusy()
            yield obj_response
            time.sleep(self.pause_for)
        self._read()
        hidebusy()
        yield obj_response

    @property
    def streaming(self):
        return True

    @property
    def style(self):
        return ''

    def _read(self, ):
        for c in 'default message':
            return c+'\n'

class CometWorker(CometBase):
    """ to use this class, subclass it, add "url"
        attribute, and define a "worker" method.
        the stdout generated by "worker" will be
        streamed, so it should probably use "print"
        or os.system("..") or whatever.  ansi color
        codes in output are also supported.
    """

    conv = Ansi2HTMLConverter(inline=False, escaped=False)

    def __init__(self, *args, **kargs):
        super(CometWorker, self).__init__(*args, **kargs)
        self.stdout = ThreadedStdout()
        self.stdout.install()

    def _make_thread(self, data):
        thr = threading.Thread(target=lambda: self.worker(**data),
                         name='testing')
        self.stdout.register(thr)
        thr.start()
        self.thr = thr
        return thr

    @property
    def style(self):
        return self.conv.get_style()

    @property
    def streaming(self):
        return self.thr.is_alive()

    def _read(self):
        return self.stdout.read_all(self.thr).replace('\n','<br/>')


    def before_streaming(self, obj_response, data):
        self._make_thread(data)


class SijaxDemo(CometWorker):
    """ This class demos how to subclass CometWorker:
        this is the minimum code you'll need to stream
        output to a given URL
    """
    url = '/comet'

    def _read(self, ):
        import random
        return random.choice('default message')

    def worker(self, **kargs):
        for x in 'hello world':
            print x,
            time.sleep(1)

class TBuff(CometBase):
    url = '/buf'

    def _read(self):
        import random
        return random.choice('default message')
